#ifndef UTILITY_ENVIRONMENT_H
#define UTILITY_ENVIRONMENT_H
/*********************************************************************************\
 *  A better-than-nothing environment and utility functions for GPGPU exercises  *
 *  (function names start with "ue_" to show that these are not part of CUDA)    *
 *                                                                               *
 *  no changes to this code (or the rest of the infrastructure) should be        *
 *  necessary to solve the exercise                                              *
 *                                                                               *
 *  based on example code of the 5.0 SDK Copyright 1993-2012 NVIDIA Corporation  *
\*********************************************************************************/

#include <cuda.h>
#include <cuda_runtime.h>
#include <device_launch_parameters.h>

#include <stdio.h>
#include <string.h>
#include <iostream>
#include <algorithm>

#include "cpu_bitmap.h"

using namespace std;

#ifdef _WIN32
#define STRCASECMP  _stricmp
#define STRNCASECMP _strnicmp
#define srand48(seed) srand(seed)
#define drand48() double(rand())/double(RAND_MAX)
#else
#define STRCASECMP  strcasecmp
#define STRNCASECMP strncasecmp
#endif


/***************************************************\
 *  General CUDA error handler                     *
 *  Prints error message, if something went wrong  *
 *  use this after every CUDA call to locate bugs  *
\***************************************************/
void handleCUDAerror( int line )
{
	cudaError_t err = cudaGetLastError();
	if ( err == cudaSuccess )
		return;

	printf("CUDA error message: %s\n"
		   "line %d\n", cudaGetErrorString(err), line );

	int devID;
    cudaGetDevice( &devID );
    cudaDeviceProp props;
    cudaGetDeviceProperties( &props, devID );
    printf("CUDA device %d: \"%s\" with compute capability %d.%d\n",
           devID, props.name, props.major, props.minor);
}


/*********************************************************\
 *  probe default CUDA device                            *
 *  prints device name and basic capabilities to stdout  *
\*********************************************************/
void ue_checkGPU(void)
{
	int numberOfDevices = 0;
	if(cudaSuccess != cudaGetDeviceCount(&numberOfDevices))
	{
		printf("No CUDA device found\n");
		exit(EXIT_FAILURE);
	}

	int deviceNumber = 0;
	if(cudaSuccess != cudaGetDevice(&deviceNumber))
	{
		printf("No CUDA device found\n");
		exit(EXIT_FAILURE);
	}

	cudaDeviceProp deviceProperties;
	if (cudaSuccess != cudaGetDeviceProperties(&deviceProperties, deviceNumber))
	{
		printf("Could not get device properties\n");
		exit(EXIT_FAILURE);
	}

	printf("using CUDA device number %d (of %d device(s) available): ", deviceNumber, numberOfDevices);
	printf("%s\n", &(deviceProperties.name));
	printf("compute capability: %0d.%0d\n", deviceProperties.major, deviceProperties.minor);
	printf("clock frequency: %d MHz, device memory: %ld MBytes\n", deviceProperties.clockRate/1000,
		deviceProperties.totalGlobalMem/(1024*1024));
	printf("device limits: %d threads\n  (%d, %d, %d) blockSize\n  (%d, %d, %d) gridSize\n", deviceProperties.maxThreadsPerBlock,
		deviceProperties.maxThreadsDim[0], deviceProperties.maxThreadsDim[1], deviceProperties.maxThreadsDim[2],
		deviceProperties.maxGridSize[0], deviceProperties.maxGridSize[1], deviceProperties.maxGridSize[2]);
	printf("\n");
}


/**********************************************************************************************\
 *  Tiny Class to measure elapsed time between cuda commands                                  *
 *  usage:                                                                                    *
 *   CudaTimer timer; // get an instance of the class                                         *
 *   timer.start(); // starts timer                                                           *
 *   // your cuda code                                                                        *
 *   timenElapsed = timer.elapsed(); // stops timer and returns elapsed time in milliseconds  *
\**********************************************************************************************/
class CudaTimer
{
public:
	CudaTimer();
	~CudaTimer();
	void start( void );
	float elapsed( void ) const;			//!< time in milliseconds

private:
	cudaEvent_t		m_start;
	cudaEvent_t		m_stop;
};

inline CudaTimer::CudaTimer()
{
	cudaEventCreate(&m_start);
	cudaEventCreate(&m_stop);
}

inline CudaTimer::~CudaTimer()
{
	cudaEventDestroy(m_start);
	cudaEventDestroy(m_stop);
}

inline void CudaTimer::start( void )
{
	cudaEventRecord(m_start);
	cudaEventSynchronize(m_start);
}

inline float CudaTimer::elapsed( void ) const
{
	cudaEventRecord(m_stop);
	cudaEventSynchronize(m_stop);

	float elapsedTime;
	cudaError_t err = cudaEventElapsedTime(&elapsedTime, m_start, m_stop);
	if ( cudaSuccess != err )
		elapsedTime = -1.0f;

	return elapsedTime;
}

#endif  // defined(UTILITY_ENVIRONMENT_H)
